from django.utils.deprecation import MiddlewareMixin
from django.urls import get_resolver, get_urlconf, NoReverseMatch, get_ns_resolver, URLResolver, URLPattern
from django.http.response import HttpResponseNotAllowed
import logging
from jhvar.django.urls import JvURLPattern


class JvRoleMiddleware(MiddlewareMixin):

    logger = logging.getLogger("jhvar.django.logger")

    def _debug(self, msg, *args, **kwargs):
        if self.logger:
            self.logger.debug(msg, *args, **kwargs)

    def process_request(self, request):
        urlconf = get_urlconf()
        resolver = get_resolver(urlconf)

        parts = request.get_full_path().split('/')
        parts.reverse()
        path = parts[:-1]
        resolved_path = []
        ns_pattern = ''
        ns_converters = {}
        while path:
            ns = path.pop()
            if ns not in resolver.app_dict:
                path.append(ns)
                break
            app_list = resolver.app_dict[ns]
            if ns not in app_list:
                ns = app_list[0]

            extra, resolver = resolver.namespace_dict[ns]
            resolved_path.append(ns)
            ns_pattern = ns_pattern + extra
            ns_converters.update(resolver.pattern.converters)

        path.reverse()
        view = "/".join(path)
        roles = None
        if isinstance(resolver, URLResolver):
            if hasattr(resolver.urlconf_module, 'permitted_roles'):
                if isinstance(resolver.urlconf_module.permitted_roles, (str, tuple, list)):
                    roles = resolver.urlconf_module.permitted_roles
            if not roles:
                for pattern in resolver.url_patterns:
                    if isinstance(pattern, URLPattern) and pattern.resolve(view):
                        if isinstance(pattern, JvURLPattern) and hasattr(pattern, 'roles'):
                            if isinstance(pattern.roles, (str, tuple, list)):
                                roles = pattern.roles
        if roles:
            if isinstance(roles, str):
                roles = [roles]
            self._debug("Url-setting requires access roles: %s" % ",".join(roles))
            if hasattr(request.session, 'roles'):
                s_roles = request.session.roles
            else:
                self._debug("There are no roles in session.roles, access denied.")
                return HttpResponseNotAllowed(permitted_methods=[])

            t = set(roles).intersection(set(s_roles))
            if len(t) == 0:
                self._debug("Session roles are '%s', do not match url-setting, access denied." % ",".join(s_roles))
                return HttpResponseNotAllowed(permitted_methods=[])




import hashlib
from uuid import uuid1
import random
import string
from django.urls import reverse
from django.utils.crypto import salted_hmac
import time


def random_key():
    uid = uuid1().hex + ''.join(random.sample(string.ascii_letters + string.digits, 32))
    hash_string = salted_hmac(time.time(), uid, secret='no_one_knows').hexdigest()
    return hash_string


def md5(s):
    m = hashlib.md5()
    # 实例化md5加密方法
    m.update(s.encode())
    # 进行加密，python2可以给字符串加密，python3只能给字节加密
    result = m.hexdigest()
    return result


def full_url(request, name, kwargs=None):
    u = reverse(name, kwargs=kwargs) if kwargs else reverse(name)
    u = request.build_absolute_uri(u)
    return u



from functools import partial
from django.urls.resolvers import RoutePattern, RegexPattern, URLPattern, URLResolver


class JvURLPattern(URLPattern):
    def __init__(self, pattern, callback, default_args=None, name=None, roles=None):
        super().__init__(pattern, callback, default_args=default_args, name=name)
        self.roles = roles

    def __repr__(self):
        return '<%s %s Roles:%s>' % (self.__class__.__name__, self.pattern.describe(), ",".join(self.roles) if self.roles else 'no-def')


def _jhvar_path(route, view, kwargs=None, name=None, Pattern=None, roles=None):
    if isinstance(view, (list, tuple)):
        # For include(...) processing.
        pattern = Pattern(route, is_endpoint=False)
        urlconf_module, app_name, namespace = view
        return URLResolver(
            pattern,
            urlconf_module,
            kwargs,
            app_name=app_name,
            namespace=namespace,
        )
    elif callable(view):
        pattern = Pattern(route, name=name, is_endpoint=True)
        return JvURLPattern(pattern, view, kwargs, name, roles)
    else:
        raise TypeError('view must be a callable or a list/tuple in the case of include().')


jv_path = partial(_jhvar_path, Pattern=RoutePattern)
jv_re_path = partial(_jhvar_path, Pattern=RegexPattern)




#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: jhvar
# Mail: jhvar@outlook.com
# Created Time:  2020-8-11 15:42:43
#############################################


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name = "jhvar-django-utils",
    version = "0.1.1",
    keywords = ("pip", "django", "role", "util"),
    description = "A toolkit for django-2.2.5 or higher",
    long_description = long_description,
    license = "MIT Licence",

    url = "https://github.com/jhvar/django-utils",
    author = "jhvar",
    author_email = "jhvar@outlook.com",

    packages = find_packages(),
    include_package_data = True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires = [
        'django>=2.2.5'
    ],
    python_requires='>=3.6',
)